@@ -10,25 +10,27 @@
 ##' @references Kvedaras V., Zemlys, V. \emph{Testing the functional constraints on parameters in regressions with variables of different frequency} Economics Letters 116 (2012) 250-254 
 ##' @examples
 ##' ##The parameter function
-##' theta.h0 <- function(p, dk, ...) {
+##' theta_h0 <- function(p, dk, ...) {
 ##'    i <- (1:dk-1)/100
 ##'    pol <- p[3]*i + p[4]*i^2
 ##'    (p[1] + p[2]*i)*exp(pol)
 ##' }
 ##'
 ##' ##Generate coefficients
-##' theta0 <- theta.h0(c(-0.1,10,-10,-10),4*12)
+##' theta0 <- theta_h0(c(-0.1,10,-10,-10),4*12)
 ##'
 ##' ##Plot the coefficients
 ##' ##Do not run
 ##' #plot(theta0)
 ##'
-##' ##Generate the predictor variable
-##' x <- simplearma.sim(list(ar=0.6),1500*12,1,12)
+##' ##' ##Generate the predictor variable
+##' xx <- ts(arima.sim(model = list(ar = 0.6), 600 * 12), frequency = 12)
 ##'
 ##' ##Simulate the response variable
-##' y <- midas.sim(500,theta0,x,1)
+##' y <- midas_sim(500, xx, theta0)
 ##'
+##' x <- window(xx, start=start(y))
+##' 
 ##' ##Create low frequency data.frame
 ##' ldt <- data.frame(y=y,trend=1:length(y))
 ##'
@@ -41,26 +43,19 @@
 ##'
 ##' ##Include intercept and trend in regression
 ##'
-##' mu.it <- midas_u(y~fmls(x,2,12)+trend, list(ldt, hdt))
+##' mu_it <- midas_u(y~fmls(x,2,12)+trend, list(ldt, hdt))
 ##'
 ##' ##Pass data as partialy named list
 ##'
-##' mu.it <- midas_u(y~fmls(x,2,12)+trend, list(ldt, x=hdt$x))
+##' mu_it <- midas_u(y~fmls(x,2,12)+trend, list(ldt, x=hdt$x))
 ##' 
 ##' @details MIDAS regression has the following form:
 ##' 
-##' \deqn{y_t=\sum_{j=0}^k\sum_{i=0}^{m-1}\theta_{jm+i} x_{(t-j)m-i}+\mathbf{z_t}\mathbf{\beta}+u_t}
-##'
-##' or alternatively
-##'
-##' \deqn{y_t=\sum_{h=0}^{(k+1)m}\theta_hx_{tm-h}+\mathbf{z_t}\mathbf{\beta}+u_t,}
-##' where \eqn{m} is the frequency ratio and
-##' \eqn{k} is the number of lags included in the regression. 
+##' \deqn{y_t = \sum_{j=1}^p\alpha_jy_{t-j} +\sum_{i=0}^{k}\sum_{j=0}^{l_i}\beta_{j}^{(i)}x_{tm_i-j}^{(i)} + u_t,}
 ##'
+##' where \eqn{x_\tau^{(i)}}, \eqn{i=0,...k} are regressors of higher (or similar) frequency than \eqn{y_t}. 
 ##' Given certain assumptions the coefficients can be estimated using usual OLS and they have the familiar properties associated with simple linear regression.
 ##'
-##' MIDAS regression involves times series with different frequencies.
-##' 
 ##' @export
 midas_u <- function(formula, data ,...) {
     Zenv <- new.env(parent=environment(formula))
@@ -81,6 +76,7 @@ midas_u <- function(formula, data ,...) {
    
     out <- eval(mf,Zenv)
     out$Zenv <- Zenv
+    out$midas_coefficients <- out$coefficients
     class(out) <- c("midas_u",class(out))
     out
 }
@@ -89,32 +85,42 @@ midas_u <- function(formula, data ,...) {
 ##'
 ##' Estimate restricted MIDAS regression using non-linear least squares.
 ##'
-##' @param x either formula for restricted MIDAS regression or \code{midas_r} object. Formula must include \code{\link{fmls}} function
+##' @param formula formula for restricted MIDAS regression or \code{midas_r} object. Formula must include \code{\link{fmls}} function
 ##' @param data a named list containing data with mixed frequencies
 ##' @param start the starting values for optimisation. Must be a list with named elements.
 ##' @param Ofunction the list with information which R function to use for optimisation. The list must have element named \code{Ofunction} which contains character string of chosen R function. Other elements of the list are the arguments passed to this function.  The default optimisation function is \code{\link{optim}} with argument \code{method="BFGS"}. Other supported functions are \code{\link{nls}}
-##' @param user.gradient the default value is FALSE, which means that the numeric approximation of weight function gradient is calculated. If TRUE  it is assumed that the R function for weight function gradient has the name of the weight function appended with \code{.gradient}. This function must return the matrix with dimensions \eqn{d_k \times q}, where \eqn{d_k} and \eqn{q} are the numbers of coefficients in unrestricted and restricted regressions correspondingly. 
+##' @param weight_gradients a named list containing gradient functions of weights. The weight gradient function must return the matrix with dimensions
+##' \eqn{d_k \times q}, where \eqn{d_k} and \eqn{q} are the number of coefficients in unrestricted and restricted regressions correspondingly.
+##' The names of the list should coincide with the names of weights used in formula.
+##' The default value is NULL, which means that the numeric approximation of weight function gradient is calculated. If the argument is not NULL, but the
+##' name of the weight used in formula is not present, it is assumed that there exists an R function which has  
+##' the name of the weight function appended with \code{_gradient}. 
 ##' @param ... additional arguments supplied to optimisation function
 ##' @return a \code{midas_r} object which is the list with the following elements:
 ##' 
 ##' \item{coefficients}{the estimates of parameters of restrictions}
-##' \item{midas.coefficientas}{the estimates of restricted coefficients of MIDAS regression}
+##' \item{midas_coefficients}{the estimates of MIDAS coefficients of MIDAS regression}
 ##' \item{model}{model data}
-##' \item{weights}{the MIDAS weights used in estimation.}
 ##' \item{unrestricted}{unrestricted regression estimated using \code{\link{midas_u}}}
-##' \item{param.map}{parameter map for optimisation function}
+##' \item{term_info}{the named list. Each element is a list with the information about the term, such as its frequency, function for weights, gradient function of weights, etc.}
 ##' \item{fn0}{optimisation function for non-linear least squares problem solved in restricted MIDAS regression}
 ##' \item{rhs}{the function which evaluates the right-hand side of the MIDAS regression}
-##' \item{allcoef}{the function which evaluates the restricted coefficients of MIDAS regression}
+##' \item{gen_midas_coef}{the function which generates the MIDAS coefficients of MIDAS regression}
 ##' \item{opt}{the output of optimisation procedure}
-##' \item{argmap.opt}{the list containing the name of optimisation function together with arguments for optimisation function}
-##' \item{start.opt}{the starting values used in optimisation}
+##' \item{argmap_opt}{the list containing the name of optimisation function together with arguments for optimisation function}
+##' \item{start_opt}{the starting values used in optimisation}
+##' \item{start_list}{the starting values as a list}
 ##' \item{call}{the call to the function}
 ##' \item{terms}{terms object}
 ##' \item{gradient}{gradient of NLS objective function}
 ##' \item{hessian}{hessian of NLS objective function}
+##' \item{gradD}{gradient function of MIDAS weight functions} 
 ##' \item{Zenv}{the environment in which data is placed}
-##' \item{user.gradient}{the value of supplied argument user.gradient}
+##' \item{use_gradient}{TRUE if user supplied gradient is used, FALSE otherwise}
+##' \item{nobs}{the number of effective observations}
+##' \item{convergence}{the convergence message}
+##' \item{fitted.values}{the fitted values of MIDAS regression}
+##' \item{residuals}{the residuals of MIDAS regression}
 ##' 
 ##' @author Virmantas Kvedaras, Vaidotas Zemlys
 ##' @references Clements, M. and Galvao, A., \emph{Macroeconomic Forecasting With Mixed-Frequency Data: Forecasting Output Growth in the United States}, Journal of Business and Economic Statistics, Vol.26 (No.4), (2008) 546-554
@@ -122,34 +128,33 @@ midas_u <- function(formula, data ,...) {
 ##' @seealso midas_r.midas_r
 ##' @examples
 ##' ##The parameter function
-##' theta.h0 <- function(p, dk, ...) {
+##' theta_h0 <- function(p, dk, ...) {
 ##'    i <- (1:dk-1)/100
 ##'    pol <- p[3]*i + p[4]*i^2
 ##'    (p[1] + p[2]*i)*exp(pol)
 ##' }
 ##'
 ##' ##Generate coefficients
-##' theta0 <- theta.h0(c(-0.1,10,-10,-10),4*12)
+##' theta0 <- theta_h0(c(-0.1,10,-10,-10),4*12)
 ##'
 ##' ##Plot the coefficients
 ##' plot(theta0)
 ##'
 ##' ##Generate the predictor variable
-##' x <- simplearma.sim(list(ar=0.6),1500*12,1,12)
+##' xx <- ts(arima.sim(model = list(ar = 0.6), 600 * 12), frequency = 12)
 ##'
 ##' ##Simulate the response variable
-##' y <- midas.sim(500,theta0,x,1)
+##' y <- midas_sim(500, xx, theta0)
 ##'
-##' ##Remove unnecessary history of x
-##' x <- window(x,start=start(y))
+##' x <- window(xx, start=start(y))
 ##' 
 ##' ##Fit restricted model
-##' mr <- midas_r(y~fmls(x,4*12-1,12,theta.h0)-1,
+##' mr <- midas_r(y~fmls(x,4*12-1,12,theta_h0)-1,
 ##'               list(y=y,x=x),
 ##'               start=list(x=c(-0.1,10,-10,-10)))
 ##'
 ##' ##Include intercept and trend in regression
-##' mr.it <- midas_r(y~fmls(x,4*12-1,12,theta.h0)+trend,
+##' mr_it <- midas_r(y~fmls(x,4*12-1,12,theta_h0)+trend,
 ##'                  list(data.frame(y=y,trend=1:500),x=x),
 ##'                  start=list(x=c(-0.1,10,-10,-10)))
 ##' 
@@ -161,25 +166,26 @@ midas_u <- function(formula, data ,...) {
 ##' trend <- 1:length(y.ar)
 ##' 
 ##' ##Fit AR(1) model
-##' mr.ar <- midas_r(y.ar ~ trend + mls(y.ar, 1, 1) +
+##' mr_ar <- midas_r(y.ar ~ trend + mls(y.ar, 1, 1) +
 ##'                  fmls(xx, 11, 12, nealmon),
 ##'                  start = list(xx = rep(0, 3)))
 ##' 
 ##' ##First order MIDAS-AR* restricted model 
-##' mr.arstar <-  midas_r(y.ar ~ trend + mls(y.ar, 1, 1, "*")
+##' mr_arstar <-  midas_r(y.ar ~ trend + mls(y.ar, 1, 1, "*")
 ##'                      + fmls(xx, 11, 12, nealmon),
 ##'                      start = list(xx = rep(0, 3)))
 ##'
 ##' @details Given MIDAS regression:
 ##'
-##' \deqn{y_t=\sum_{j=0}^k\sum_{i=0}^{m-1}\theta_{jm+i} x_{(t-j)m-i}+\mathbf{z_t}\beta+u_t}
-##'
+##' \deqn{y_t = \sum_{j=1}^p\alpha_jy_{t-j} +\sum_{i=0}^{k}\sum_{j=0}^{l_i}\beta_{j}^{(i)}x_{tm_i-j}^{(i)} + u_t,}
+##' 
 ##' estimate the parameters of the restriction
 ##'
-##' \deqn{\theta_h=g(h,\lambda),}
-##' where \eqn{h=0,...,(k+1)m}, together with coefficients \eqn{\beta} corresponding to additional low frequency regressors.
+##' \deqn{\beta_j^{(i)}=g^{(i)}(j,\lambda).}
 ##'
-##' MIDAS regression involves times series with different frequencies. 
+##' Such model is a generalisation of so called ADL-MIDAS regression. It is not required that all the coefficients should be restricted, i.e the function \eqn{g^{(i)}}
+##' might be an identity function. Model with no restrictions is called U-MIDAS model. The regressors \eqn{x_\tau^{(i)}} must be of higher
+##' (or of the same) frequency as the dependent variable \eqn{y_t}. 
 ##'
 ##' MIDAS-AR* (a model with a common factor, see (Clements and Galvao, 2008)) can be estimated by specifying additional argument, see an example.
 ##'
@@ -187,15 +193,9 @@ midas_u <- function(formula, data ,...) {
 ##' the MIDAS regression.
 ##' 
 ##' @export
-midas_r <- function(x,...)UseMethod("midas_r")
+midas_r <- function(formula, data, start, Ofunction="optim", weight_gradients=NULL,...) {
 
-
-#' @rdname midas_r
-#' @method midas_r default
-#' @export
-midas_r.default <- function(x, data, start, Ofunction="optim", user.gradient=FALSE,...) {
-
-    Zenv <- new.env(parent=environment(x))
+    Zenv <- new.env(parent=environment(formula))
 
     if(missing(data)) {
         ee <- NULL
@@ -204,13 +204,17 @@ midas_r.default <- function(x, data, start, Ofunction="optim", user.gradient=FAL
         ee <- data_to_env(data)
         parent.env(ee) <- parent.frame()
     }
+
+    if(missing(start)) {
+        stop("Please supply starting values.")
+    } 
     
     assign("ee",ee,Zenv)
-    x <- as.formula(x)
+    formula <- as.formula(formula)
     cl <- match.call()    
     mf <- match.call(expand.dots = FALSE)
-    mf$x <- x
-    m <- match(c("x", "data"), names(mf), 0L)
+    mf$formula <- formula
+    m <- match(c("formula", "data"), names(mf), 0L)
     mf <- mf[c(1L, m)]
     mf[[1L]] <- as.name("model.frame")
     mf[[3L]] <- as.name("ee")   
@@ -227,65 +231,97 @@ midas_r.default <- function(x, data, start, Ofunction="optim", user.gradient=FAL
     y <- model.response(mf, "numeric")
     X <- model.matrix(mt, mf)
     
-    prepmd <- prepmidas_r(y,X,mt,Zenv,cl,args,start,Ofunction,user.gradient,itr$lagsTable)
+    prepmd <- prepmidas_r(y,X,mt,Zenv,cl,args,start,Ofunction,weight_gradients,itr$lagsTable)
     
     class(prepmd) <- "midas_r"
     midas_r.fit(prepmd)    
 }
 
-##' Restricted MIDAS regression
-##'
-##' Reestimate the MIDAS regression with different starting values
-##' 
-##' @param x \code{midas_r} object 
-##' @param start the starting values
-##' @param Ofunction a character string of the optimisation function to use. The default value is to use the function of previous optimisation.
-##' @param ... further arguments to optimisation function. If none are supplied, the arguments of previous optimisation are used.
-##' @return \code{midas_r} object
-##' @method midas_r midas_r
-##' @seealso midas_r
-##' @author Vaidotas Zemlys
+##' @method update midas_r
 ##' @export
-midas_r.midas_r <- function(x,start=coef(x),Ofunction=x$argmap.opt$Ofunction,...) {
-   
-    oarg <- list(...)
-    cl <- match.call()
-    dotargnm <- names(oarg)
-    
-    ##Perform check whether arguments are ok and eval them
-    if(length(dotargnm)>0) {
-        offending <- dotargnm[!dotargnm %in% names(formals(Ofunction))]
-        if(length(offending)>0)  {
-            stop(paste("The function ",Ofunction," does not have the following arguments: ", paste(offending,collapse=", "),sep=""))
-        }
-    }
-    else {
-        oarg <- NULL
+update.midas_r <- function(object, formula.,..., evaluate = TRUE) {
+    if (is.null(call <- getCall(object))) 
+        stop("need an object with call component")
+    extras <- match.call(expand.dots = FALSE)$...
+    if (!missing(formula.)) 
+        call$formula <- update.formula(formula(object), formula.)
+          
+    if (length(extras)) {
+        existing <- !is.na(match(names(extras), names(call))) 
+        for (a in names(extras)[existing]) call[[a]] <- extras[[a]]
+        if (any(!existing)) {
+            call <- c(as.list(call), extras[!existing])
+            call <- as.call(call)
+        }        
     }
 
-    if(Ofunction!=x$argmap.opt$Ofunction) {
-        argmap <- c(list(Ofunction=Ofunction),oarg)
-    }              
-    else {
-         argmap <- x$argmap.opt
-         argmap$Ofunction <- NULL
-         argnm <- union(names(argmap),names(oarg))
-         marg <- vector("list",length(argnm))
-         names(marg) <- argnm
-         ##New supplied arguments override the old ones
-         marg[names(oarg)] <- oarg
-         ##Already set arguments are left intact
-         oldarg <- setdiff(names(argmap),names(oarg))
-         marg[oldarg] <- argmap[oldarg]
-         argmap <- c(list(Ofunction=Ofunction),marg)
+    ##1. If no start is supplied update the start from the call
+    ##2. If start is supplied intersect it with already fitted values.
+   
+    cf <- coef(object)
+    ustart <- lapply(object$term_info,function(x)cf[x[["coef_index"]]])
+
+    redo <- FALSE
+    if(!("start" %in% names(extras))) {        
+        if(!("start" %in% names(call) && is.null(call$start))) {
+            call$start <- ustart
+            object$start_opt <- cf
+        }
+    } else {
+        if(is.null(extras$start)) {
+            ##If start is null, we want to fit unrestricted midas model, this means that we need to call midas_r
+            call["start"] <- list(NULL)                        
+            redo <- TRUE
+        } else {
+            cstart <- eval(call$start,object$Zenv)
+            ustart[names(cstart)] <- cstart
+            call$start <- ustart
+            object$start_opt <- unlist(ustart)
+        }
+    }        
+    if (evaluate) {
+        if(!missing(formula.) || "data" %in% names(extras)  || "weight_gradients" %in% names(extras) || redo) {
+            eval(call, parent.frame())
+        } else {
+            ##If we got here, we assume that we do not need to reevaluate terms.
+            if(!is.null(extras$Ofunction)) {
+                Ofunction <- eval(extras$Ofunction)
+                extras$Ofunction <- NULL
+            } else Ofunction <- object$argmap_opt$Ofunction            
+            dotargnm <- names(extras)
+            if (length(dotargnm) > 0) {
+                offending <- dotargnm[!dotargnm %in% names(formals(Ofunction))]
+                if (length(offending) > 0) {
+                    stop(paste("The function ", Ofunction, " does not have the following arguments: ", 
+                               paste(offending, collapse = ", "), sep = ""))
+                }
+            }
+            else {
+                extras <- NULL
+            }
+            if (Ofunction != object$argmap_opt$Ofunction) {
+                argmap <- c(list(Ofunction = Ofunction), extras)
+            }
+            else {
+                argmap <- object$argmap_opt
+                argmap$Ofunction <- NULL
+                argnm <- union(names(argmap), names(extras))
+                marg <- vector("list", length(argnm))
+                names(marg) <- argnm
+                marg[names(extras)] <- extras
+                oldarg <- setdiff(names(argmap), names(extras))
+                marg[oldarg] <- argmap[oldarg]
+                argmap <- c(list(Ofunction = Ofunction), marg)
+            }
+            object$call <- call
+            object$argmap_opt <- argmap
+            midas_r.fit(object)
+        }
     }
-    
-    x$start.opt <- start
-    x$argmap.opt <- argmap
-    x$call <- cl
-    midas_r.fit(x)
+    else call
 }
 
+
 ##' Fit restricted MIDAS regression
 ##'
 ##' Workhorse function for fitting restricted MIDAS regression
@@ -294,13 +330,13 @@ midas_r.midas_r <- function(x,start=coef(x),Ofunction=x$argmap.opt$Ofunction,...
 ##' @return \code{\link{midas_r}} object
 ##' @author Vaidotas Zemlys
 midas_r.fit <- function(x) {
-    args <- x$argmap.opt
+    args <- x$argmap_opt
     function.opt <- args$Ofunction
     args$Ofunction <- NULL
     if(function.opt=="optim" | function.opt=="spg") {  
-        args$par <- x$start.opt
+        args$par <- x$start_opt
         args$fn <- x$fn0
-        if(x$user.gradient) {
+        if(x$use_gradient) {
             args$gr <- x$gradient
         }
         opt <- try(do.call(function.opt,args),silent=TRUE)
@@ -312,9 +348,9 @@ midas_r.fit <- function(x) {
         x$convergence <- opt$convergence
     }
     if(function.opt=="optimx") {  
-        args$par <- x$start.opt
+        args$par <- x$start_opt
         args$fn <- x$fn0
-        if(x$user.gradient) {
+        if(x$use_gradient) {
             args$gr <- x$gradient
         }
         opt <- try(do.call(function.opt,args),silent=TRUE)
@@ -335,7 +371,7 @@ midas_r.fit <- function(x) {
     }
     if(function.opt=="nls") {
         rhs <- x$rhs
-        if(x$user.gradient) {
+        if(x$use_gradient) {
             orhs <- rhs
             rhs <- function(p) {
                 res <- orhs(p)
@@ -345,7 +381,7 @@ midas_r.fit <- function(x) {
         }
         y <- x$model[,1]
         args$formula <- formula(y~rhs(p))
-        args$start <- list(p=x$start.opt)
+        args$start <- list(p=x$start_opt)
         opt <- try(do.call("nls",args),silent=TRUE)
         if(inherits(opt,"try-error")) {
             stop("The optimisation algorithm of MIDAS regression failed with the following message:\n", opt,"\nPlease try other starting values or a different optimisation function")
@@ -357,32 +393,37 @@ midas_r.fit <- function(x) {
     x$opt <- opt
     x$coefficients <- par
     names(par) <- NULL
-    x$midas.coefficients <- x$allcoef(par)
-    x$fitted.values <- as.vector(x$model[,-1]%*%x$midas.coefficients)
+    x$midas_coefficients <- x$gen_midas_coef(par)
+    x$fitted.values <- as.vector(x$model[,-1]%*%x$midas_coefficients)
     x$residuals <- as.vector(x$model[,1]-x$fitted.values)
     x
 }
 
-##' Prepare necessary objects for fitting of the MIDAS regression
-##'
-##' Prepare necessary objects for fitting of the MIDAS regression
-##' @param y the response
-##' @param X the model matrix
-##' @param mt the terms of the formula
-##' @param Zenv the environment to evaluate the formula
-##' @param cl call of the function
-##' @param args additional argument
-##' @param start starting values
-##' @param Ofunction the optimisation function
-##' @param user.gradient see \link{midas_r} documentation
-##' @param lagsTable the lagstable from \link{checkARstar}
-##' @param unrestricted the unrestricted model
-##' @param guess_start if TRUE, get the initial values for non-MIDAS terms via OLS, if FALSE, initialize them with zero.
-##' @author Vaidotas Zemlys
-prepmidas_r <- function(y,X,mt,Zenv,cl,args,start,Ofunction,user.gradient,lagsTable,unrestricted=NULL,guess_start=TRUE) {
-    
-    ##High frequency variables can enter to formula
-    ##only within fmls function
+## Prepare necessary objects for fitting of the MIDAS regression
+##
+## y the response
+## X the model matrix
+## mt the terms of the formula
+## Zenv the environment to evaluate the formula
+## cl call of the function
+## args additional argument
+## start starting values
+## Ofunction the optimisation function
+## weight_gradients a list of gradient functions for weights
+## lagsTable the lagstable from checkARstar
+## unrestricted the unrestricted model
+## guess_start if TRUE, get the initial values for non-MIDAS terms via OLS, if FALSE, initialize them with zero.
+## Vaidotas Zemlys
+prepmidas_r <- function(y, X, mt, Zenv, cl, args, start, Ofunction, weight_gradients, lagsTable, unrestricted = NULL, guess_start = TRUE) {
+
+    start <- start[!sapply(start,is.null)]
+    if(is.null(weight_gradients)) use_gradient <- FALSE
+    else use_gradient=TRUE
+
+    if(!is.null(args$guess_start)) {
+        guess_start <- args$guess_start
+        args$guess_start <- NULL
+    }    
     terms.lhs <- as.list(attr(mt,"variables"))[-2:-1]
     term.labels <- attr(mt,"term.labels") 
 
@@ -393,10 +434,17 @@ prepmidas_r <- function(y,X,mt,Zenv,cl,args,start,Ofunction,user.gradient,lagsTa
          mf[[1]] <- fr[[5]]
          noarg <- length(formals(eval(fr[[5]],Zenv)))
          if(noarg<2)stop("The weight function must have at least two arguments")
+         freq <- mf[[4]]         
          mf <- mf[1:min(length(mf),noarg+1)]
          for(j in 3:length(mf)) {
              mf[[j]] <- eval(mf[[j]],Zenv)
-         }        
+         }
+         lagstruct <- switch(type,
+                              fmls = 0:mf[[3]],
+                              dmls = 0:mf[[3]],
+                              mls = mf[[3]]
+                              )
+        
          mf[[3]] <- switch(type,
                            fmls = mf[[3]]+1,
                            dmls = mf[[3]]+1, 
@@ -405,30 +453,63 @@ prepmidas_r <- function(y,X,mt,Zenv,cl,args,start,Ofunction,user.gradient,lagsTa
              mf[[2]] <- p
              eval(mf,Zenv)
          }
-         gmf <- mf
-         ##Make this customizable
-         gmf[[1]] <- as.name(paste0(as.character(fr[[5]]),".gradient"))
-         grf <- function(p) {
-             gmf[[2]] <- p
-             eval(gmf,Zenv)
-         }
+
+         if(use_gradient) {
+             gmf <- mf
+             weight_name <- as.character(fr[[5]])
+             if(weight_name %in% names(weight_gradients)) {
+                 weight_gradient_name <- paste0(as.character(fr[[2]]),"_tmp_gradient_fun")
+                 gmf[[1]] <- as.name(weight_gradient_name)
+                 assign(weight_gradient_name,weight_gradients[[weight_name]],Zenv)
+             } else {
+                 #warning("Gradient function for weight ", weight_name, " is not present, using function ",weight_name,"_gradient for gradient")
+                 gmf[[1]] <- as.name(paste0(weight_name,"_gradient"))
+             }
+             grf <- function(p) {
+                 gmf[[2]] <- p
+                 eval(gmf,Zenv)
+             }
+         } else grf <- NULL
          return(list(weight=rf,
-                     name=as.character(fr[[2]]),
+                     term_name=as.character(fr[[2]]),
                      gradient=grf,
-                     start=rep(0,mf[[3]]),
-                     wlabel=as.character(fr[[2]])))
+                     start=rep(0,mf[[3]]),                    
+                     weight_name=as.character(fr[[5]]),
+                     frequency=freq,
+                     lag_structure=lagstruct
+                     )
+                
+                )
     }
     
-    uterm <- function(name,k=1) {
+    uterm <- function(name, k=1, lags, frequency) {
         force(k)
         list(weight=function(p)p,
-             name=name,
+             term_name=name,
              gradient=function(p)diag(k),
-             start=rep(0,k),
-             wlabel="")
+             start=rep(0,k),             
+             weight_name="",
+             frequency = frequency,
+             lag_structure = lags)
         
     }
 
+    wuterm <- function(fr, fun) {
+        lags <- eval(fr[[3]],Zenv)
+        nol <- switch(fun,
+                      fmls = lags+1,
+                      dmls = lags+1,
+                      mls = length(lags)
+                      )
+        lagstruct <- switch(fun,
+                            fmls = 0:lags,
+                            dmls = 0:lags,
+                            mls = lags
+                            )
+        freq <- eval(fr[[4]],Zenv)
+        nm <- as.character(fr[[2]])
+        uterm(nm,nol,lagstruct,frequency=freq)
+    }
     
     for(i in 1:length(rfd)) {
         fr <- terms.lhs[[i]]
@@ -437,33 +518,34 @@ prepmidas_r <- function(y,X,mt,Zenv,cl,args,start,Ofunction,user.gradient,lagsTa
             if(length(fr)>=5 && fr[[5]] != "*") {
                 wterm(fr,fun)
             } else {
-                lags <- eval(fr[[3]],Zenv)
-                nol <- switch(fun,
-                              fmls = lags+1,
-                              dmls = lags+1,
-                              mls = length(lags)
-                              )
-                nm <- as.character(fr[[2]])
-                uterm(nm,nol)
-            }            
+                wuterm(fr,fun)
+            }                         
         }
         else {
-            uterm(term.labels[i],1)
+            uterm(term.labels[i],1,0,1)
         }
     }
    
     if (attr(mt,"intercept")==1)  {
-        rfd <- c(list(list(weight=function(p)p,name="(Intercept)",gradient=function(p)return(matrix(1)),start=0,wlabel="")),rfd)
+        rfd <- c(list(list(weight=function(p)p,
+                           term_name="(Intercept)",
+                           gradient=function(p)return(matrix(1)),
+                           start=0,
+                           weight_name="",
+                           frequency = 1,
+                           lag_structure = 0
+                           )),rfd)
         term.labels <- c("(Intercept)",term.labels)
     }
     
     rf <- lapply(rfd,"[[","weight")
-    names(rf) <- sapply(rfd,"[[","name")
+    names(rf) <- sapply(rfd,"[[","term_name")
     
-
-    weight_names <- sapply(rfd,"[[","wlabel")
+    ##Note this is a bit of misnomer. Variable weight_names is actualy a vector of term names which have MIDAS weights.
+    ##It *is not* the same as actual name of weight function. This is a left-over from the old code. 
+    weight_names <- sapply(rfd,"[[","weight_name")
     weight_inds <- which(weight_names!="")
-    weight_names <- weight_names[weight_names!=""]
+    weight_names <- names(rf)[weight_names!=""]
     
     
     start_default <- lapply(rfd,"[[","start")
@@ -472,14 +554,17 @@ prepmidas_r <- function(y,X,mt,Zenv,cl,args,start,Ofunction,user.gradient,lagsTa
     ##If there are no weight functions, we have unrestricted MIDAS model.
     if(length(weight_names)==0)Ofunction <- "lm"
     else {
-        if(any(!weight_names%in% names(start)))stop("Starting values for weight hyperparameters must be supplied")
+        if(is.null(start)) {            
+            cl$formula <- update_weights(cl$formula,setNames(lapply(1:length(weight_names), function(x)""), weight_names))
+            warning("Since the start = NULL, it is assumed that U-MIDAS model is fitted")
+            return(eval(cl,Zenv))            
+        } else {
+            if(any(!weight_names%in% names(start)))stop("Starting values for weight parameters must be supplied")
+        }
     }
     
     start_default[names(start)] <- start
-
-    
-    #restr.no <- sum(sapply(start_default[weight_names], length))
-    
+            
     np <- cumsum(sapply(start_default,length))
 
     build_indices <- function(ci,nm) {
@@ -491,12 +576,16 @@ prepmidas_r <- function(y,X,mt,Zenv,cl,args,start,Ofunction,user.gradient,lagsTa
     }
     
     pinds <- build_indices(np,names(start_default))
-    
+
     for(i in 1:length(start_default))names(start_default[[i]]) <- NULL
 
+    initial_midas_coef <- mapply(function(fun,st)fun(st),rf,start_default,SIMPLIFY=FALSE)
+    if(sum(is.na(unlist(initial_midas_coef)))>0) stop("Check your starting values, NA in midas coefficients") 
+    
+    npx <- cumsum(sapply(initial_midas_coef,length))
+    xinds <- build_indices(npx,names(start_default))        
+     
     if(length(weight_names)>0 && guess_start) {
-        npx <- cumsum(sapply(mapply(function(fun,st)fun(st),rf,start_default,SIMPLIFY=FALSE),length))
-        xinds <- build_indices(npx,names(start_default))
         wi <- rep(FALSE,length(rf))
         wi[weight_inds] <- TRUE
         Xstart <- mapply(function(fun,st,inds,iswhgt) {        
@@ -562,7 +651,7 @@ prepmidas_r <- function(y,X,mt,Zenv,cl,args,start,Ofunction,user.gradient,lagsTa
         sum(r^2)
     }
 
-    if(!user.gradient) {
+    if(!use_gradient) {
         gradD <- function(p)jacobian(all_coef,p)
         gr <- function(p)grad(fn0,p)
     }
@@ -615,9 +704,9 @@ prepmidas_r <- function(y,X,mt,Zenv,cl,args,start,Ofunction,user.gradient,lagsTa
                 } else if(j == dind) {
                   hfv <- !sapply(lagsTable, is.null)
                   for(k in which(hfv)) {
-                    for(cl in 1:ncol(mlv) + 1) {
+                    for(ccl in 1:ncol(mlv) + 1) {
                       ltb <- lagsTable[[k]]
-                      mlv[pindm[[k]]$row, cl - 1] <- mlv[ind$row, cl - 1] + rf[[k]](pp[[k]]) * ltb[rowSums(ltb) != 0, cl]
+                      mlv[pindm[[k]]$row, ccl - 1] <- mlv[ind$row, ccl - 1] + rf[[k]](pp[[k]]) * ltb[rowSums(ltb) != 0, ccl]
                     }
                   }
                 }
@@ -639,36 +728,50 @@ prepmidas_r <- function(y,X,mt,Zenv,cl,args,start,Ofunction,user.gradient,lagsTa
     
     hess <- function(x)numDeriv::hessian(fn0,x)
       
-    if(is.null(unrestricted)) {
-        if(ncol(X)<nrow(X)) unrestricted <- lm(y~.-1,data=data.frame(cbind(y,X),check.names=FALSE))
+    if(is.null(unrestricted)) {        
+        if(ncol(X)<nrow(X)) {
+            if(attr(mt,"intercept")==1) {
+                unrestricted <- lm(y~.,data=data.frame(cbind(y,X[,-1]),check.names=FALSE))
+            } else {
+                unrestricted <- lm(y~.-1,data=data.frame(cbind(y,X),check.names=FALSE))
+            }
+            
+        }
     }
 
     control <- c(list(Ofunction=Ofunction),args)
     ##Override default method of optim. Use BFGS instead of Nelder-Mead
     if(!("method"%in% names(control)) & Ofunction=="optim") {        
         control$method <- "BFGS"
-    }
-    
+    }    
+    term_info <- rfd
+    names(term_info) <- sapply(term_info,"[[","term_name")
+    term_info <- mapply(function(term,pind,xind){
+        term$start <- NULL
+        term$coef_index <- pind
+        term$midas_coef_index <- xind
+        term
+    },term_info,pinds[names(term_info)],xinds[names(term_info)],SIMPLIFY=FALSE)
+
     list(coefficients=starto,
-         midas.coefficients=all_coef(starto),
-         model=cbind(y,X),
-         weights=rf[weight_inds],
+         midas_coefficients=all_coef(starto),
+         model=cbind(y,X),         
          unrestricted=unrestricted,
-         param.map=pinds,
+         term_info=term_info,
          fn0=fn0,
          rhs=mdsrhs,
-         allcoef=all_coef,
+         gen_midas_coef = all_coef,
          opt=NULL,
-         argmap.opt=control,
-         start.opt=starto,
-         start.list=start,
+         argmap_opt=control,
+         start_opt=starto,
+         start_list=start,
          call=cl,
          terms=mt,
          gradient=gr,
          hessian=hess,
          gradD=gradD,
          Zenv=Zenv,
-         user.gradient=user.gradient,
+         use_gradient=use_gradient,
          nobs=nrow(X))   
 }
 
@@ -763,11 +866,10 @@ midas_r_simple <- function(y,X,z=NULL,weight,grw=NULL,startx,startz=NULL,method=
     call <- match.call()
     fitted.values <- as.vector(XX%*%all_coef(par))
     list(coefficients=par,
-         midas.coefficients=all_coef(par),
+         midas_coefficients=all_coef(par),
          model=model,
          weights=weight,
-         fn0=fn0,
-         allcoef=all_coef,
+         fn0=fn0,    
          opt=opt,
          call=call,
          gradient=gr,
@@ -777,12 +879,42 @@ midas_r_simple <- function(y,X,z=NULL,weight,grw=NULL,startx,startz=NULL,method=
          residuals=as.vector(y-fitted.values))
              
 }
-
-##' Check whether the MIDAS model is MIDAS-AR* model
+##' Updates weights in a expression with MIDAS term
 ##'
-##' Checks whether the MIDAS model is MIDAS-AR* model and provides necessary modifications
-##' @param trms terms of the model formula
-##' @author Julius Vainora
+##' For a MIDAS term \code{fmls(x, 6, 1, nealmon)} change weight \code{nealmon} to another weight.
+##' @title Updates weights in MIDAS regression formula
+##' @param expr expression with MIDAS term
+##' @param tb a named list with redefined weights
+##' @return an expression with changed weights
+##' @author Vaidotas Zemlys
+##' @export
+##' @examples
+##'
+##' update_weights(y~trend+mls(x,0:7,4,nealmon)+mls(z,0:16,12,nealmon),list(x = "nbeta", z = ""))
+##' 
+update_weights <- function(expr,tb) {
+    if(length(expr)==3) {
+        expr[[2]] <- update_weights(expr[[2]],tb)
+        expr[[3]] <- update_weights(expr[[3]],tb)
+    }
+    if(length(expr)==5) {
+        fun <- as.character(expr[[1]])
+        if(fun[[1]] %in% c("fmls","mls","dmls")) {
+            term_name <- as.character(expr[[2]])
+            if(term_name %in% names(tb)) {
+                if(is.null(tb[[term_name]])|| tb[[term_name]] == "") {
+                    expr <- expr[1:4]
+                } else expr[[5]] <- as.name(tb[[term_name]])                    
+            }
+        }
+        else return(expr)
+    }
+    return(expr)
+}
+
+## Check whether the MIDAS model is MIDAS-AR* model
+##
+## authored by Julius Vainora
 checkARstar <- function(trms) {
   vars <- as.list(attr(trms, "variables"))[-2:-1]
   env <- environment(trms)
