@@ -26,7 +26,7 @@ rvhk <- function(h,k){
     y <- y[1:length(rvh)]
     mu <- midas_u(rvh~fmls(y,k,1))
     cfur <- coef(mu)[grep("fmls",names(coef(mu)))]
-    midas_r(midas_r(rvh~fmls(y,k,1,nlmn),start=list(y=prestart(c(0.2,-1,1),cfur,k+1))),Ofunction="nls")   
+    update(midas_r(rvh~fmls(y,k,1,nlmn),start=list(y=prestart(c(0.2,-1,1),cfur,k+1))),Ofunction="nls")   
 }
 
 allh <- lapply(c(5,10,20,40),rvhk,k=69)
@@ -47,33 +47,23 @@ lapply(allh,summary)
 PHI <- lapply(allh,function(x)meatHAC(x$unrestricted,prewhite=TRUE,weights=weightsAndrews))
 
 ###Apply hAh test
-lapply(allh,hAh.test)
+lapply(allh,hAh_test)
 
 ##Apply robust hAh test with precomputed PHI
-mapply(hAhr.test,allh,PHI,SIMPLIFY=FALSE)
+mapply(hAhr_test,allh,PHI,SIMPLIFY=FALSE)
 
 ##Parameter j is superfluous, j=0 means no logarithm transformation was
 ##applied, j=1 means that logarithm transformation was applied. The graph
 ##is made to be the same as in the aforementioned article.
 
-graph <- function(x,phi,j,h) {
-    cfur <- coef(x$unrestricted)
-    cfur <- cfur[grep("fmls",names(cfur))]
-    cfre <- weight_coef(x)
-    k <- length(cfur)
-    sdval <- sqrt(diag(sandwich(x$unrestricted,meat=phi)))
-    sdval <- sdval[grep("fmls",names(sdval))]
-    pv0hac <- hAhr.test(x,PHI=phi)$p.value
-    plot(c(0:(k - 1)), c(cfur), col = "black", ylab = "Beta coefficients", xlab = "h")
-    title(main = sprintf("k(H=%.0f,j=%.0f) = %.0f: p-val.(hAh_HAC) < %.2f", h, j, 
-        k, max(pv0hac, 0.01)), cex.main = 1, font.main = 4, col.main = "black")
-    
-    points(c(0:(k - 1)), cfre[1:k], type = "l", col = "blue")
-    points(c(0:(k - 1)), cfur[1:k] + 2 * sdval[1:k], type = "l", col = "red", lty = 2)
-    points(c(0:(k - 1)), cfur[1:k] - 2 * sdval[1:k], type = "l", col = "red", lty = 2)
+graph <- function(x,phi,j,h) {    
+    k <- length(coef(x, midas=TRUE, term_names = "y"))
+    pv0hac <- hAhr_test(x,PHI=phi)$p.value
+    ttl <- sprintf("k(H=%.0f,j=%.0f) = %.0f: p-val.(hAh_HAC) < %.2f", h, j, k, max(pv0hac, 0.01))
+    plot_midas_coef(x, title = ttl, term_name = "y")
 }
 
 dev.new()
 par(mfrow=c(2,2))
 
-mapply(graph,allh,PHI,as.list(rep(1,4)),as.list(c(5,10,20,40)),SIMPLIFY=FALSE)
+plot_info <- mapply(graph,allh,PHI,as.list(rep(1,4)),as.list(c(5,10,20,40)),SIMPLIFY=FALSE)
